const mailService = require('pubsweet-component-mail-service')
const logger = require('@pubsweet/logger')
const helpers = require('./helpers')

module.exports = {
  sendSignupEmail: async ({ dashboardUrl, res, email, UserModel }) => {
    let user
    try {
      user = await UserModel.findByEmail(email)
    } catch (e) {
      const notFoundError = await helpers.handleNotFoundError(e, 'User')
      return res.status(notFoundError.status).json({
        error: notFoundError.message,
      })
    }
    if (!user.confirmationToken) {
      return res
        .status(400)
        .json({ error: 'User does not have a confirmation token.' })
    }
    try {
      await mailService.sendSimpleEmail({
        toEmail: user.email,
        user,
        emailType: 'signup',
        dashboardUrl,
        meta: {
          confirmationToken: user.confirmationToken,
        },
      })
      return res.status(200).json({})
    } catch (e) {
      logger.error(e)
      return res.status(500).json({ error: 'Email could not be sent.' })
    }
  },
  setupNewUserEmail: async ({ dashboardUrl, res, email, role, UserModel }) => {
    let user
    try {
      user = await UserModel.findByEmail(email)
    } catch (e) {
      const notFoundError = await helpers.handleNotFoundError(e, 'User')
      return res.status(notFoundError.status).json({
        error: notFoundError.message,
      })
    }
    if (!user.passwordResetToken) {
      return res
        .status(400)
        .json({ error: 'User does not have a password reset token.' })
    }
    let emailType
    switch (role) {
      case 'handlingEditor':
      case 'editorInChief':
      case 'admin':
        emailType = 'invite'
        break
      case 'author':
        emailType = 'invite-author'
        break
      default:
        return res.status(400).json({
          error: `Role ${role} cannot be used with a password reset email.`,
        })
    }
    try {
      await mailService.sendSimpleEmail({
        toEmail: user.email,
        user,
        emailType,
        dashboardUrl,
      })
      return res.status(200).json({})
    } catch (e) {
      logger.error(e)
      return res.status(500).json({ error: 'Email could not be sent.' })
    }
  },
}
