const {
  Email,
  services,
  Fragment,
  Collection,
  Invitation,
} = require('pubsweet-component-helper-service')

module.exports = models => async (req, res) => {
  const { collectionId, invitationId, fragmentId } = req.params
  const { isAccepted, reason } = req.body

  const UserModel = models.User
  const user = await UserModel.find(req.user)
  try {
    const collection = await models.Collection.find(collectionId)
    if (!collection.fragments.includes(fragmentId))
      return res.status(400).json({
        error: `Fragment ${fragmentId} does not match collection ${collectionId}`,
      })
    const fragment = await models.Fragment.find(fragmentId)
    fragment.invitations = fragment.invitations || []
    const invitation = await fragment.invitations.find(
      invitation => invitation.id === invitationId,
    )

    const invitationHelper = new Invitation({
      userId: user.id,
      role: 'reviewer',
    })
    const invitationValidation = invitationHelper.validateInvitation({
      invitation,
    })
    if (invitationValidation.error)
      return res.status(invitationValidation.status).json({
        error: invitationValidation.error,
      })

    const collectionHelper = new Collection({ collection })
    const fragmentHelper = new Fragment({ fragment })
    const parsedFragment = await fragmentHelper.getFragmentData({
      handlingEditor: collection.handlingEditor,
    })
    const baseUrl = services.getBaseUrl(req)
    const {
      authorsList: authors,
      submittingAuthor,
    } = await fragmentHelper.getAuthorData({ UserModel })
    const emailHelper = new Email({
      UserModel,
      collection,
      parsedFragment,
      baseUrl,
      authors,
    })

    invitation.respondedOn = Date.now()
    invitation.hasAnswer = true
    if (isAccepted) {
      invitation.isAccepted = true
      if (collection.status === 'reviewersInvited')
        await collectionHelper.updateStatus({ newStatus: 'underReview' })
      await fragment.save()

      emailHelper.setupReviewerDecisionEmail({
        agree: true,
        timestamp: invitation.respondedOn,
        user,
        authorName: `${submittingAuthor.firstName} ${
          submittingAuthor.lastName
        }`,
      })

      return res.status(200).json(invitation)
    }

    invitation.isAccepted = false
    if (reason) invitation.reason = reason
    await fragment.save()

    collectionHelper.updateStatusByNumberOfReviewers({
      invitations: fragment.invitations,
    })

    emailHelper.setupReviewerDecisionEmail({
      agree: false,
      user,
    })

    return res.status(200).json(invitation)
  } catch (e) {
    const notFoundError = await services.handleNotFoundError(e, 'Item')
    return res.status(notFoundError.status).json({
      error: notFoundError.message,
    })
  }
}
