const logger = require('@pubsweet/logger')
const { services } = require('pubsweet-component-helper-service')
const mailService = require('pubsweet-component-mail-service')

module.exports = models => async (req, res) => {
  const { email } = req.body
  if (!services.checkForUndefinedParams(email))
    return res.status(400).json({ error: 'Email address is required.' })

  try {
    const user = await models.User.findByEmail(email)
    if (user.passwordResetTimestamp) {
      const resetDate = new Date(user.passwordResetTimestamp)
      const hoursPassed = Math.floor(
        (new Date().getTime() - resetDate) / 3600000,
      )
      if (hoursPassed < 24) {
        return res
          .status(400)
          .json({ error: 'A password reset has already been requested.' })
      }
    }

    user.passwordResetToken = generatePasswordHash()
    user.passwordResetTimestamp = Date.now()
    await user.save()

    mailService.sendSimpleEmail({
      toEmail: user.email,
      user,
      emailType: 'forgot-password',
      dashboardUrl: services.getBaseUrl(req),
    })
  } catch (e) {
    logger.error(
      `A forgot password request has been made on an non-existent email: ${email}`,
    )
  }

  res.status(200).json({
    message: `A password reset email has been sent to ${email}.`,
  })
}

const generatePasswordHash = () =>
  Array.from({ length: 4 }, () =>
    Math.random()
      .toString(36)
      .slice(4),
  ).join('')
